<?php

/*********************************************************************

 PHP Search Ver 1.00 (2010/12/23)

*********************************************************************/

/********* ライセンス ************************************************

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 @Link http://www.php-labo.net/
 @Copyright(C) 2007-2010 PHP Labo
 @Author Knight <info at php-labo dot net>

*********************************************************************/

/********* 外部ファイル読み込み *************************************/

require_once './smarty/Smarty.class.php';
require_once './config.php';

if (PICTOGRAM_MODE) {
	require_once PICTOGRAM_MAIN_FILE;
}

/********* メイン処理 ***********************************************/

//ユーザー情報取得
$users = user();

//セッション開始
session();

//データ正規化
normalize();

//設定読み込み
$configs = config();

//URLルーティング
$params = routing();

//テンプレート設定
$smarty = template();

//プラグイン実行
plugin('page');

//モードに応じた処理を実行
switch ($_REQUEST['mode']) {
	case 'admin':
		if (isloggedin()) {
			switch ($_REQUEST['work']) {
				case 'config':
					admin_config();
					break;
				case 'config_edit':
					admin_config_edit();
					break;
				case 'logout':
					admin_logout();
					break;
				default:
					admin_config();
			}
		} else {
			admin();
		}
		break;
	default:
		search();
}

exit;

/********* ユーザー定義関数 *****************************************/

/* ユーザー情報取得 */
function user()
{
	$users = array();

	//接続環境取得
	if (preg_match('/(DoCoMo|SoftBank|Vodafone|J-PHONE|UP\.Browser|DDIPOCKET|WILLCOM)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['type'] = 'mobile';
	} else {
		$users['type'] = 'pc';
	}

	//キャリア取得
	if (preg_match('/DoCoMo/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'docomo';
	} elseif (preg_match('/(SoftBank|Vodafone|J-PHONE)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'softbank';
	} elseif (preg_match('/UP\.Browser/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'au';
	} elseif (preg_match('/(DDIPOCKET|WILLCOM)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'willcom';
	} else {
		$users['career'] = null;
	}

	//コンテンツタイプ取得
	if (preg_match('/(DoCoMo|SoftBank|Vodafone|J-PHONE|DDIPOCKET|WILLCOM)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['content'] = 'application/xhtml+xml';
	} else {
		$users['content'] = 'text/html';
	}

	//フォームメソッド取得
	if (preg_match('/(J-PHONE|UP\.Browser)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['method'] = 'get';
	} else {
		$users['method'] = 'post';
	}

	//固体識別情報取得
	if (isset($_SERVER['HTTP_X_DCMGUID'])) {
		$users['serial'] = $_SERVER['HTTP_X_DCMGUID'];
	} elseif (isset($_SERVER['HTTP_X_UP_SUBNO'])) {
		$users['serial'] = $_SERVER['HTTP_X_UP_SUBNO'];
	} elseif (isset($_SERVER['HTTP_X_JPHONE_UID'])) {
		$users['serial'] = $_SERVER['HTTP_X_JPHONE_UID'];
	} else {
		$users['serial'] = null;
	}

	return $users;
}

/* セッション開始 */
function session()
{
	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_PATH);
	} else {
		$info = parse_url(HTTP_PATH);
	}

	session_set_cookie_params(SESSION_LIFETIME, $info['path']);
	session_cache_limiter(SESSION_CACHE);
	session_start();

	return;
}

/* 受信データ正規化 */
function normalize()
{
	global $users;

	//不正データ削除
	$_GET     = sanitize($_GET);
	$_POST    = sanitize($_POST);
	$_REQUEST = sanitize($_REQUEST);
	$_SERVER  = sanitize($_SERVER);
	$_COOKIE  = sanitize($_COOKIE);

	//改行コード統一
	$_GET     = unify($_GET);
	$_POST    = unify($_POST);
	$_REQUEST = unify($_REQUEST);
	$_SERVER  = unify($_SERVER);
	$_COOKIE  = unify($_COOKIE);

	//絵文字コード統一
	if (PICTOGRAM_MODE) {
		$_GET  = unify_pictogram($_GET);
		$_POST = unify_pictogram($_POST);
	}

	//文字コード変換
	if ($users['career'] == 'au') {
		$_GET     = convert($_GET);
		$_POST    = convert($_POST);
		$_REQUEST = convert($_REQUEST);
		$_SERVER  = convert($_SERVER);
	}

	//GETデータ取得
	if ($users['method'] == 'get') {
		$_POST = $_GET;
	}

	return;
}

/* 設定読み込み */
function config()
{
	$configs = array();

	if ($dir = scandir(CONFIG_DIR)) {
		foreach ($dir as $entry) {
			if (!is_file(CONFIG_DIR . $entry) or !preg_match('/^(\w+)\.ini$/', $entry, $matches)) {
				continue;
			}

			$config = parse_ini_file(CONFIG_DIR . $entry);

			foreach ($config as $key => $value) {
				$value = str_replace('\\\\', "\a", $value);
				$value = str_replace('\n', "\n", $value);
				$value = str_replace("\a", '\\', $value);

				$configs[$matches[1]][$key] = $value;
			}
		}
	} else {
		exit('設定ファイル格納ディレクトリを開けません。');
	}

	return $configs;
}

/* URLルーティング */
function routing()
{
	$request_uri = explode('/', strtok($_SERVER['REQUEST_URI'], '?'));
	$script_name = explode('/', $_SERVER['SCRIPT_NAME']);

	for ($i = 0; $i < sizeof($script_name); $i++) {
		if ($request_uri[$i] == $script_name[$i]) {
			unset($request_uri[$i]);
		}
	}
	$params = array_values(array_map('urldecode', $request_uri));

	if (isset($params[0]) and preg_match('/\D/', $params[0])) {
		if (!isset($_REQUEST['mode'])) {
			$_REQUEST['mode'] = $params[0];
		}
		if (!isset($_REQUEST['work'])) {
			$_REQUEST['work'] = '';
		}
	} else {
		if (!isset($_REQUEST['mode'])) {
			$_REQUEST['mode'] = '';
		}
		if (!isset($_REQUEST['work'])) {
			$_REQUEST['work'] = '';
		}
	}

	return $params;
}

/* テンプレート設定 */
function template()
{
	global $users, $configs;

	$smarty = new Smarty();
	$smarty->template_dir = TEMPLATE_DIR;
	$smarty->compile_dir  = TEMPLATE_COMPILE;

	$smarty->default_modifiers = array('escape');

	if (REWRITE_MODE) {
		$http_url  = preg_replace('/\/$/', '', HTTP_PATH);
		$https_url = preg_replace('/\/$/', '', HTTPS_PATH ? HTTPS_PATH : HTTP_PATH);
	} else {
		$http_url  = HTTP_PATH . MAIN_FILE;
		$https_url = (HTTPS_PATH ? HTTPS_PATH : HTTP_PATH) . MAIN_FILE;
	}

	if ($users['serial'] and ADMIN_SERIAL and in_array($users['serial'], explode(',', ADMIN_SERIAL))) {
		$login = true;
	} elseif (!empty($_SESSION['login']) or !empty($_COOKIE['session'])) {
		$login = true;
	} else {
		$login = false;
	}

	$smarty->assign('info', array(
		'session_name' => session_name(),
		'session_id'   => session_id(),
		'title'        => $configs['_search']['title'],
		'back_url'     => $configs['_search']['back_url'],
		'configs'      => $configs,
		'http_path'    => HTTP_PATH,
		'https_path'   => HTTPS_PATH ? HTTPS_PATH : HTTP_PATH,
		'main_file'    => MAIN_FILE,
		'http_url'     => $http_url,
		'https_url'    => $https_url,
		'login'        => $login,
		'type'         => $users['type'],
		'career'       => $users['career'],
		'content'      => $users['content'],
		'method'       => $users['method'],
		'serial'       => $users['serial']
	));

	return $smarty;
}

/* プラグイン実行 */
function plugin($type)
{
	if ($dir = scandir(PLUGIN_DIR)) {
		foreach ($dir as $entry) {
			if (!is_file(PLUGIN_DIR . $entry) or !preg_match("/^$type\.\w+\.php$/", $entry)) {
				continue;
			}

			include_once PLUGIN_DIR . $entry;
		}
	} else {
		exit('プラグイン格納ディレクトリを開けません。');
	}

	return;
}

/* 検索画面 */
function search()
{
	global $users, $configs, $params, $smarty;

	//検索語句取得
	if (isset($params[1])) {
		$word = preg_replace('/\+/', ' ', $params[1]);
	} else {
		$word = isset($_GET['word']) ? $_GET['word'] : '';
	}
	$smarty->assign('word', $word);

	//プラグイン実行
	plugin('begin');

	//検索実行
	if ($word) {
		$files = search_dir($configs['_search']['target_dir'], $word);

		$results = array();
		foreach ($files as $result) {
			if ($configs['_search']['title_format'] != '' and preg_match('/' . $configs['_search']['title_format'] . '/', $result['title'], $matches)) {
				$result['title'] = $matches[1];
			}

			$content = implode('', $result['contents']);
			$url     = preg_replace('/' . preg_quote($configs['_search']['target_dir'], '/') . '/', $configs['_search']['target_url'], $result['path']);

			$results[] = array(
				'path'    => $result['path'],
				'title'   => $result['title'],
				'content' => $content,
				'url'     => $url
			);
		}

		$smarty->assign('results', $results);
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_SEARCH);

	//プラグイン実行
	plugin('end');

	return;
}

/* ログイン状態チェック */
function isloggedin()
{
	global $users, $configs, $params, $smarty;

	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_PATH);
	} else {
		$info = parse_url(HTTP_PATH);
	}

	if ($users['serial'] and ADMIN_SERIAL) {
		//固体識別情報確認
		if (in_array($users['serial'], explode(',', ADMIN_SERIAL))) {
			return true;
		}
	} elseif (isset($_POST['pwd'])) {
		//パスワード認証
		if ($_POST['pwd'] == ADMIN_PWD) {
			$_SESSION['login'] = true;

			if (isset($_POST['session'])) {
				$session = md5(uniqid(rand(), true));

				if (touch(LOGIN_DIR . $session)) {
					chmod(LOGIN_DIR . $session, 0644);
				} else {
					exit('ログイン情報を保持できません。');
				}

				setcookie('session', $session, time() + COOKIE_EXPIRE, $info['path']);
			}
		}

		header('Location: ' . (HTTPS_PATH ? HTTPS_PATH : HTTP_PATH) . MAIN_FILE . '?mode=admin&work=default');

		exit;
	} elseif (empty($_SESSION['login'])) {
		//ログイン状態を復元
		if (isset($_COOKIE['session'])) {
			if ($dir = scandir(LOGIN_DIR)) {
				foreach ($dir as $entry) {
					if (is_file(LOGIN_DIR . $entry) and time() - filemtime(LOGIN_DIR . $entry) > COOKIE_EXPIRE) {
						unlink(LOGIN_DIR . $entry);
					}
				}
			} else {
				exit('ログイン情報格納ディレクトリを開けません。');
			}

			if (file_exists(LOGIN_DIR . $_COOKIE['session'])) {
				$_SESSION['login'] = true;

				$session = md5(uniqid(rand(), true));

				if (rename(LOGIN_DIR . $_COOKIE['session'], LOGIN_DIR . $session) and touch(LOGIN_DIR . $session)) {
					setcookie('session', $session, time() + COOKIE_EXPIRE, $info['path']);
				} else {
					exit('ログイン情報を保持できません。');
				}
			}
		}
	}

	if (isset($_SESSION['login']) and $_SESSION['login'] == true) {
		return true;
	} else {
		return false;
	}
}

/* 管理者用 - 認証画面 */
function admin()
{
	global $users, $configs, $params, $smarty;

	//プラグイン実行
	plugin('begin');

	//エラーメッセージ取得
	if ($_REQUEST['work'] == 'default') {
		$smarty->append('errors', 'パスワードが違います。');
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 設定一覧 */
function admin_config()
{
	global $users, $configs, $params, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', create_token());

	//作業対象ファイル取得
	$file = isset($_REQUEST['file']) ? $_REQUEST['file'] : '_search';

	//プラグイン実行
	plugin('begin');

	//設定一覧取得
	$explain = '本体';
	$files[] = array(
		'name'    => '_search',
		'explain' => $explain,
		'flag'    => '_search' == $file ? true : false
	);
	$target = $explain;

	if ($dir = scandir(CONFIG_DIR)) {
		foreach ($dir as $entry) {
			if (!preg_match('/^(\w+)\.ini$/', $entry, $matches) or $matches[1] == '_search') {
				continue;
			}

			$name    = $matches[1];
			$explain = $matches[1] . 'プラグイン';

			if ($matches[1] == $file) {
				$target = $explain;
			}

			$files[] = array(
				'name'    => $name,
				'explain' => $explain,
				'flag'    => $matches[1] == $file ? true : false
			);
		}
	} else {
		exit('設定ファイル格納ディレクトリを開けません。');
	}

	$smarty->assign(array(
		'files'  => $files,
		'target' => $target
	));

	//設定内容取得
	$flag   = false;
	$buffer = '';
	$count  = 0;
	$types  = array();
	$sizes  = array();
	$exts   = array();
	$datas  = array();

	if ($fp = fopen(CONFIG_DIR . $file . '.ini', 'r')) {
		while ($line = fgets($fp)) {
			$line = trim($line);

			if ($buffer and (preg_match('/^\[.+\]$/', $line) or preg_match('/^;.+$/', $line))) {
				$datas[] = array(
					'type' => 'config',
					'data' => '<textarea name="data[_buffer' . ++$count . ']" cols="50" rows="5">' . $buffer . '</textarea>'
				);

				$buffer = '';
			}

			if (preg_match('/^\[(.+)\]$/', $line, $matches)) {
				$flag = true;

				$datas[] = array(
					'type' => 'section',
					'data' => $matches[1]
				);
			} elseif (preg_match('/^;(.+)$/', $line, $matches)) {
				if ($matches[1] == 'exit') {
					break;
				}

				if ($flag) {
					$datas[] = array(
						'type' => 'comment',
						'data' => $matches[1]
					);
				} else {
					$define = $matches[1];

					if (preg_match('/^(.+)\s+(.+)\s+"(.+)"$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = $matches[3];
					} elseif (preg_match('/^(.+)\s+(.+)$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = '';
					} else {
						$name = '';
						$type = '';
						$ext  = '';
					}

					if ($name) {
						if (preg_match('/^(.+)\((\d+)\)$/', $type, $matches)) {
							$type = $matches[1];
							$size = $matches[2];
						} else {
							$size = '';
						}

						$types[$name] = $type;
						$sizes[$name] = $size;
						$exts[$name]  = $ext;
					}
				}
			} elseif (preg_match('/^([^\s]+)\s*=\s*(.+)$/', $line, $matches)) {
				if (!$flag) {
					continue;
				}

				$name  = $matches[1];
				$value = $matches[2];

				$value = str_replace('<', '&lt;', $value);
				$value = str_replace('>', '&gt;', $value);

				if (empty($types[$name])) {
					$buffer .= $line . "\n";
				} else {
					$data = '';

					switch ($types[$name]) {
						case 'set':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$value = $matches[1];
							}

							$options = explode('|', $exts[$name]);

							foreach ($options as $option) {
								list($option_key, $option_value) = explode(':', $option, 2);

								if (in_array($option_key, explode(',', $value))) {
									$checked = ' checked="checked"';
								} else {
									$checked = '';
								}

								$data .= '<input type="checkbox" name="data[' . $name . '][]" id="' . $name . '_' . $option_key . '" value="' . $option_key . '"' . $checked . ' /> <label for="' . $name . '_' . $option_key . '"">' . $option_value . '</label><br />';
							}

							break;
						case 'enum':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$value = $matches[1];
							}

							$options = explode('|', $exts[$name]);

							foreach ($options as $option) {
								list($option_key, $option_value) = explode(':', $option, 2);

								if ($value == $option_key) {
									$checked = ' checked="checked"';
								} else {
									$checked = '';
								}

								$data .= '<input type="radio" name="data[' . $name . ']" id="' . $name . '_' . $option_key . '" value="' . $option_key . '"' . $checked . ' /> <label for="' . $name . '_' . $option_key . '"">' . $option_value . '</label><br />';
							}

							break;
						case 'bool':
							if (empty($exts[$name])) {
								$options = array('On', 'Off');
							} else {
								$options = explode('|', $exts[$name], 2);
							}

							$data .= '<select name="data[' . $name . ']">';
							if ($value == 'On') {
								$data .= '<option value="On" selected="selected">' . $options[0] . '</option>';
								$data .= '<option value="Off">' . $options[1] . '</option>';
							} else {
								$data .= '<option value="On">' . $options[0] . '</option>';
								$data .= '<option value="Off" selected="selected">' . $options[1] . '</option>';
							}
							$data .= '</select>';

							break;
						case 'text':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$matches[1] = preg_replace('/" CONFIG_QUOTE "/', '&quot;', $matches[1]);
								$matches[1] = str_replace('\\\\', "\a", $matches[1]);
								$matches[1] = str_replace('\n', "\n", $matches[1]);
								$matches[1] = str_replace("\a", '\\', $matches[1]);

								$data .= '<textarea name="data[' . $name . ']" cols="50" rows="5">' . $matches[1] . '</textarea>';
							}

							break;
						case 'char':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$matches[1] = preg_replace('/" CONFIG_QUOTE "/', '&quot;', $matches[1]);
								$matches[1] = str_replace('\\\\', '\\', $matches[1]);

								$data .= '<input type="text" name="data[' . $name . ']" size="' . (empty($sizes[$name]) ? 80 : $sizes[$name]) . '" value="' . $matches[1] . '" />';
							}

							if (!empty($exts[$name])) {
								$data .= ' ' . $exts[$name];
							}

							break;
						case 'int':
							$data .= '<input type="text" name="data[' . $name . ']" size="' . (empty($sizes[$name]) ? 10 : $sizes[$name]) . '" value="' . $value . '" />';

							if (!empty($exts[$name])) {
								$data .= ' ' . $exts[$name];
							}

							break;
					}

					$datas[] = array(
						'type' => 'config',
						'data' => $data
					);
				}
			}
		}
		fclose($fp);
	} else {
		exit('設定ファイルを読み込めません。');
	}

	$smarty->assign('form', array(
		'file'  => $file,
		'datas' => $datas
	));

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_CONFIG);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 設定編集 */
function admin_config_edit()
{
	global $users, $configs, $params, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!check_token()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_POST['file'])) {
		error('設定対象が指定されていません。');
	}
	if (!preg_match('/^[\w\-]+$/', $_POST['file'])) {
		error('設定対象の指定内容が正しくありません。');
	}

	//プラグイン実行
	plugin('begin');

	//設定編集
	$flag1 = false;
	$flag2 = false;
	$data  = '';
	$count = 0;
	$types = array();
	$sizes = array();
	$exts  = array();

	if ($fp = fopen(CONFIG_DIR . $_POST['file'] . '.ini', 'r')) {
		while ($line = fgets($fp)) {
			$line = trim($line);

			if (preg_match('/^\[(.+)\]$/', $line, $matches)) {
				$flag1 = true;

				$data .= $line . "\n";
			} elseif (preg_match('/^;(.+)$/', $line, $matches)) {
				if (!$flag1) {
					$define = $matches[1];

					if (preg_match('/^(.+)\s+(.+)\s+"(.+)"$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = $matches[3];
					} elseif (preg_match('/^(.+)\s+(.+)$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = '';
					} else {
						$name = '';
						$type = '';
						$ext  = '';
					}

					if ($name) {
						if (preg_match('/^(.+)\((\d+)\)$/', $type, $matches)) {
							$type = $matches[1];
							$size = $matches[2];
						} else {
							$size = '';
						}

						$types[$name] = $type;
						$sizes[$name] = $size;
						$exts[$name]  = $ext;
					}
				}
				if ($flag2) {
					$flag2 = false;
				}

				$data .= $line . "\n";
			} elseif (preg_match('/^([^\s]+)\s*=\s*(.+)$/', $line, $matches)) {
				if (!$flag1) {
					continue;
				}

				$name  = $matches[1];
				$value = $matches[2];

				if (empty($types[$name])) {
					if ($flag2) {
						continue;
					}

					if (isset($_POST['data']['_buffer' . ($count + 1)])) {
						$value = trim($_POST['data']['_buffer' . ++$count]);

						if (preg_match('/(^|\n);/', $value)) {
							error('設定内容の行頭に;を書くことはできません。');
						}

						$flag2 = true;
					} else {
						$value = $line;
					}

					$data .= trim($value) . "\n";
				} else {
					switch ($types[$name]) {
						case 'set':
							if (isset($_POST['data'][$name])) {
								$value = '"' . implode(',', $_POST['data'][$name]) . '"';
							} else {
								$value = '""';
							}
							break;
						case 'enum':
							if (isset($_POST['data'][$name])) {
								$value = '"' . $_POST['data'][$name] . '"';
							}
							break;
						case 'bool':
							if (isset($_POST['data'][$name])) {
								$value = $_POST['data'][$name];
							}
							if (!preg_match('/^(On|Off)$/', $value)) {
								error($name . 'はOn/Offで入力してください。');
							}
							break;
						case 'text':
							if (isset($_POST['data'][$name])) {
								$value = '"' . preg_replace('/"/', '" CONFIG_QUOTE "', $_POST['data'][$name]) . '"';
							}
							break;
						case 'char':
							if (isset($_POST['data'][$name])) {
								$value = '"' . preg_replace('/"/', '" CONFIG_QUOTE "', $_POST['data'][$name]) . '"';
							}
							if (preg_match('/\n/', $value)) {
								error($name . 'は一行で入力してください。');
							}
							break;
						case 'int':
							if (isset($_POST['data'][$name])) {
								$value = $_POST['data'][$name];
							}
							if (!preg_match('/^\d+$/', $value)) {
								error($name . 'は数値で入力してください。');
							}
							break;
					}

					$value = str_replace('\\', '\\\\', $value);
					$value = str_replace("\n", '\n', $value);

					$data .= $name . ' = ' . trim($value) . "\n";
				}
			} else {
				$data .= $line . "\n";
			}
		}
		fclose($fp);
	} else {
		exit('設定ファイルを読み込めません。');
	}

	if (!file_put_contents(CONFIG_DIR . $_POST['file'] . '.ini', $data)) {
		exit('設定ファイルに書き込めません。');
	}

	$smarty->assign('message', '設定ファイルを編集しました。');

	//ワンタイムトークン破棄
	destroy_token();

	//プラグイン実行
	plugin('end');

	//設定一覧
	admin_config();

	return;
}

/* 管理者用 - ログアウト */
function admin_logout()
{
	global $users, $configs, $params, $smarty;

	//プラグイン実行
	plugin('begin');

	//ログイン情報クリア
	if (isset($_COOKIE['session'])) {
		unlink(LOGIN_DIR . $_COOKIE['session']);

		if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
			$info = parse_url(HTTPS_PATH);
		} else {
			$info = parse_url(HTTP_PATH);
		}

		setcookie('session', '', 0, $info['path']);
	}
	$_SESSION['login'] = false;

	//プラグイン実行
	plugin('end');

	//管理画面
	admin();

	return;
}

/* エラー表示 */
function error($message)
{
	global $users, $configs, $params, $smarty;

	$smarty->assign('message', $message);

	output(TEMPLATE_ERROR);

	exit;
}

/* データ出力 */
function output($template, $id = null, $content = null)
{
	global $users, $configs, $params, $pdo, $smarty;

	if ($content == null) {
		$content = $users['content'];
	}

	header('Content-Type: ' . $content . '; charset=UTF-8');

	if ($users['type'] == 'mobile') {
		header('Pragma: no-cache');
		header('Cache-Control: no-cache');

		if ($smarty->template_exists(TEMPLATE_MOBILE . $template)) {
			$template = TEMPLATE_MOBILE . $template;
		}
	}

	if (PICTOGRAM_MODE) {
		echo convert_pictogram($smarty->fetch($template, $id));
	} else {
		$smarty->display($template, $id);
	}

	return;
}

/* ディレクトリ内検索 */
function search_dir($path, $word)
{
	global $configs;

	$files = array();

	if (!file_exists($path)) {
		return $files;
	}

	if ($dir = scandir($path)) {
		$tmp_directories = array();
		$tmp_files       = array();

		foreach ($dir as $data) {
			if ($data == '.' or $data == '..') {
				continue;
			}

			if (is_dir($path . $data)) {
				$tmp_directories[] = $data;
			} elseif (is_file($path . $data)) {
				$tmp_files[] = $data;
			}
		}

		$dir = array_merge($tmp_files, $tmp_directories);
	}

	foreach ($dir as $data) {
		if ($data == '.' or $data == '..') {
			continue;
		}
		if (is_dir($path . $data)) {
			if ($configs['_search']['forbidden_dir'] and preg_match('/' . $configs['_search']['forbidden_dir'] . '/', $data)) {
				continue;
			}

			$results = search_dir($path . $data . '/', $word);

			if (!empty($results)) {
				$files = array_merge($files, $results);
			}
		} elseif (is_file($path . $data)) {
			if ($configs['_search']['target_file'] and !preg_match('/' . $configs['_search']['target_file'] . '/', $data)) {
				continue;
			}

			$results = search_file($path . $data, $word);

			if (!empty($results)) {
				$files[] = $results;
			}
		}
	}

	return $files;
}

/* ファイル内検索 */
function search_file($path, $word)
{
	global $configs;

	$keywords = explode(' ', str_replace('　', ' ', $word));
	$hits     = array();

	$title    = '';
	$contents = array();

	if ($fp = fopen($path, 'r')) {
		while ($line = fgets($fp)) {
			if ($title == '' and preg_match('/<title>(.+)<\/title>/', $line, $matches)) {
				$title = $matches[1];
			}

			$line = strip_tags($line);

			$flag = false;
			foreach ($keywords as $keyword) {
				if (preg_match('/' . preg_quote($keyword, '/') . '/', $line)) {
					$hits[$keyword] = true;
					$flag           = true;
				}
			}

			if ($flag) {
				$contents[] = trim($line);
			}
		}

		fclose($fp);
	}

	$flag = true;
	foreach ($keywords as $keyword) {
		if (!isset($hits[$keyword])) {
			$flag = false;
		}
	}

	if ($flag) {
		return array(
			'path'     => $path,
			'title'    => $title,
			'contents' => $contents
		);
	} else {
		return array();
	}
}

/* ワンタイムトークン作成 */
function create_token()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return;
	}

	$token = md5(uniqid(rand(), true));

	$_SESSION['token'] = $token;

	return $token;
}

/* ワンタイムトークン比較 */
function check_token()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return true;
	}

	if (isset($_REQUEST['token']) and isset($_SESSION['token']) and $_REQUEST['token'] == $_SESSION['token']) {
		return true;
	} else {
		return false;
	}
}

/* ワンタイムトークン破棄 */
function destroy_token()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return;
	}

	$_SESSION['token'] = '';

	return;
}

/* 不正データ削除 */
function sanitize($data)
{
	if (is_array($data)) {
		return array_map('sanitize', $data);
	}

	return str_replace("\0", '', $data);
}

/* 改行コード統一 */
function unify($data)
{
	if (is_array($data)) {
		return array_map('unify', $data);
	}

	$data = preg_replace("/\r?\n/", "\r", $data);
	$data = preg_replace("/\r/", "\n", $data);

	return $data;
}

/* 文字コード変換 */
function convert($data)
{
	if (is_array($data)) {
		return array_map('convert', $data);
	}

	return mb_convert_encoding($data, 'UTF-8', 'UTF-8,SJIS-WIN');
}

/* 絵文字コード統一 */
function unify_pictogram($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('unify_pictogram', $data);
	}

	if ($users['career'] == 'docomo' or $users['career'] == 'softbank' or $users['career'] == 'au') {
		if ($users['career'] == 'docomo') {
			$career  = MPC_FROM_FOMA;
			$charset = MPC_FROM_CHARSET_UTF8;
		} elseif ($users['career'] == 'softbank') {
			$career  = MPC_FROM_SOFTBANK;
			$charset = MPC_FROM_CHARSET_UTF8;
		} elseif ($users['career'] == 'au') {
			$career  = MPC_FROM_EZWEB;
			$charset = MPC_FROM_CHARSET_SJIS;
		}
		$mpc = MobilePictogramConverter::factory($data, $career, $charset, MPC_FROM_OPTION_RAW);
	} else {
		$mpc = MobilePictogramConverter::factory($data, MPC_FROM_FOMA, MPC_FROM_CHARSET_UTF8, MPC_FROM_OPTION_WEB);
	}
	$mpc->setImagePath(PICTOGRAM_IMAGE_DIR);

	return $mpc->Convert(MPC_TO_FOMA, MPC_TO_OPTION_RAW);
}

/* キャリアに応じた絵文字に変換 */
function convert_pictogram($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('convert_pictogram', $data);
	}

	$mpc = MobilePictogramConverter::factory($data, MPC_FROM_FOMA, MPC_FROM_CHARSET_UTF8, MPC_FROM_OPTION_RAW);
	$mpc->setImagePath(PICTOGRAM_IMAGE_DIR);

	$string = $mpc->autoConvert();

	if ($users['career'] != 'docomo' and $users['career'] != 'softbank' and $users['career'] != 'au') {
		$string = escape_pictogram($string);
		$string = str_replace('$', "\a", $string);
		$string = preg_replace('/(value="([^"\\\\]|\\\\.)*")/e', 'unescape_pictogram("$1","text")', $string);
		$string = preg_replace('/(<textarea [^>]+>[^<]+<\/textarea>)/e', 'unescape_pictogram("$1","text")', $string);
		$string = str_replace("\a", '$', $string);
		$string = unescape_pictogram($string);
	}

	return $string;
}

/* 絵文字コード削除 */
function except_pictogram($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('except_pictogram', $data);
	}

	$mpc = MobilePictogramConverter::factory($data, MPC_FROM_FOMA, MPC_FROM_CHARSET_UTF8, MPC_FROM_OPTION_RAW);
	$mpc->setImagePath(PICTOGRAM_IMAGE_DIR);

	return $mpc->Except();
}

/* 絵文字をエスケープ */
function escape_pictogram($data)
{
	return preg_replace('/<img src="([^"]+)" alt="" border="0" width="12" height="12" \/>/', '[&&$1]', $data);
}

/* 絵文字をアンエスケープ */
function unescape_pictogram($data, $type = 'html')
{
	if ($type == 'html') {
		return preg_replace('/\[&&([^\]]+)\]/', '<img src="$1" alt="" border="0" width="12" height="12" />', $data);
	} else {
		return preg_replace('/\[&&([^\]]+)\]/e', '"&amp;#".basename("$1",".gif").";"', $data);
	}
}

?>
